import configparser  # 导入配置文件解析模块
import pygame
from random import randrange

file = 'config.ini'  # 配置文件路径
config = configparser.ConfigParser()  # 创建解析器对象
config.read(file, encoding='UTF-8')  # 读取并解析配置文件


class GameSprite(pygame.sprite.Sprite):  # 定义游戏对象的抽象类
    def __init__(self, image):  # 定义构造方法（需要添加图片参数）
        pygame.sprite.Sprite.__init__(self)  # 重载超类的构造方法
        self.image = pygame.image.load(image).convert_alpha()  # 创建image特性存储Surface对象
        self.rect = self.image.get_rect()  # 通过image创建rect特性
        screen = pygame.display.get_surface()  # 获取屏幕外观
        shrink = -config.getint('Screen', 'margin')  # 获取配置文件数值
        self.area = screen.get_rect().inflate(shrink, shrink)  # 获取屏幕外观填充后的区域（飞机的活动范围）


class Enemy(GameSprite):  # 定义敌方飞机对象类
    def __init__(self, speed):  # 定义构造方法（需要添加速度参数）
        GameSprite.__init__(self, config.get('Image', 'enemy'))  # 重载超类构造方法（参数中传入图片）
        self.speed = speed  # 为敌方飞机设置速度特性
        self.reset()  # 重置敌方飞机位置

    def reset(self):  # 定义重置位置的方法
        x = randrange(self.area.left, self.area.right)  # 从活动范围中获取x轴的随机位置
        self.rect.midbottom = x, 0  # 设置游戏对象底部中心位置

    def update(self):  # 定义刷新位置的方法
        self.rect.top += self.speed  # 设置每帧向下移动的距离
        self.landed = self.rect.top >= self.area.bottom  # 获取敌机是否已从屏幕下方完全移出


class Plane(GameSprite):  # 定义己方飞机对象类
    def __init__(self):  # 定义构造方法
        GameSprite.__init__(self, config.get('Image', 'plane'))  # 重载超类构造方法（参数中传入图片）
        self.rect.bottom = self.area.bottom  # 设置游戏对象垂直方向底部初始位置在屏幕底部
        self.pad_top = config.getint('Plane_padding', 'top')  # 获取配置文件中游戏对象顶部填充数值
        self.pad_side = config.getint('Plane_padding', 'side')  # 获取配置文件中游戏对象两侧填充数值

    def update(self):  # 定义刷新位置的方法
        self.rect.centerx = pygame.mouse.get_pos()[0]  # 设置游戏对象水平中心位置为鼠标指针的x轴位置
        self.rect = self.rect.clamp(self.area)  # 设置游戏对象水平方向可活动区域为屏幕外观填充后的区域

    def touches(self, other):  # 设置触碰到其它游戏对象的方法
        bounds = self.rect.inflate(-self.pad_side, -self.pad_top)  # 获取游戏对象填充后的边界范围（顶部与两侧）
        bounds.bottom = self.rect.bottom  # 设置边界范围的底部为游戏对象的底部
        return bounds.colliderect(other.rect)  # 返回游戏对象边界是否触碰到其它游戏对象
